// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "OmSourceAddSourcesDlg.h"
#include "OmSourcePhotometryDlg.h"

#include <qapplication.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qdialog.h>
#include <qtable.h>
#include <qstringlist.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qtooltip.h>
#include "OmSourceVersionNo.h"
#include "OmSourceParameters.h"
#include <iostream>
#include "OmSourceButton.h"
#include "OmSpinBox.h"
#include "OmSourceGroupBox.h"
#include "OmSourceLabel.h"
#include "OmSourceNumerics.h"
#include "OmImageDisplay.h"
#include "OmSourceRegions.h"
#include "OmSourceProcessDlg.h"
#include "OmSourceUtility.h"
#include "OmSourceHelp.h" 
#include "average.h"
#include <cmath>

F90MODPROCPROTO(void,omsource_glue, deletebackgroundlist)();
F90MODPROCPROTO(void, omsource_glue, deletebackgroundregions)();
F90MODPROCPROTO(void, omsource_glue, initialisephotometry)();
F90MODPROCPROTO(void, omsource_glue, addregions1)(OmSourceRegions*);
F90MODPROCPROTO(void, omsource_glue, addregions2)(OmSourceRegions*);
F90MODPROCPROTO(void, omsource_glue, centroidregions)(OmSourceRegions*);



F90MODPROCPROTO(void, omsource_glue, addbackgroundregions)(OmSourceRegions *,const float &);
F90MODPROCPROTO(void, omsource_glue, getthecounts)();
F90MODPROCPROTO(void, omsource_glue, appendtosourceregions)(OmSourceRegions *);
F90MODPROCPROTO(float, omsource_glue, getimagecounts)(float &, float&, float&);
F90MODPROCPROTO(void, omsource_glue, computesourceparameters)(OmSourceParameters &sources);


F90MODPROCPROTO(void, omsource_glue, computephotometry )(OmSourceParameters &sources);
F90MODPROCPROTO(void, omsource_glue, computephotometry2)(OmSourceParameter &source);

F90MODPROCPROTO(void, omsource_glue, computesourcepositions2)(OmSourceParameter &source);


F90MODPROCPROTO(void, omsource_glue, appendtosourcelist)(OmSourceParameters &sources);


extern F90MODVARDEF(OmSourceParameters, omsource_glue, mybackgroundsources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, myaddedsources);

extern F90MODVARDEF(OmSourceRegions, omsource_glue, mybackgroundregions);


extern F90MODVARDEF(int, omsource_glue, myimagenx);
extern F90MODVARDEF(int, omsource_glue, myimageny);



/********************************************************************************
 * Dialog for obtaining background regions from the user          
 *
 *******************************************************************************/




/************************************************************************************
*
************************************************************************************/
AddSourcesDlg::AddSourcesDlg(QWidget *parent,  OmImageDisplay *imageDisplay,
    float apertureRadius) :
    QDialog(parent, "", true), _imageDisplay(imageDisplay),
    _apertureRadius(apertureRadius)

{


    _browser = (OmSourceBrowser *)parent;
 
  

    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    if (mySources.ubound() > 0) 
        _oldSources = true;
    else
       _oldSources = false;
 

    // Need the image dimensions to stop the user selecting sources outside the image
    _nx = (float)(F90MODPROC( omsource_glue, myimagenx ) + 1);
    _ny = (float)(F90MODPROC( omsource_glue, myimageny ) + 1);
   


    _timer = 0; 
    _label = new OmSourceLabel( this, " test ", 20, 30, 315, 110);



    //    cout << "Line 93\n";
    updateTimeChanged( 3 );
   


    int maxWidth = _label->geometry().right() + 20;

  



    // Create the table for storing details about the selected background regions 
    _table = new QTable(10, 6, this, 0);
    _table->setGeometry(20, 150, 300, 120);
  
    int noColumns = 7;

    _table->setReadOnly(true);   // Can only read the table
    _table->setNumCols( noColumns);
    _table->setNumRows(1);   
   
    QHeader *th = _table->horizontalHeader();
    QToolTip::add(_table, "This table stores the following information about the sources you selected-\n\n"
                           "Column 1- the x coordinate (pixels)\n"
                           "Column 2- the y coordinate (pixels)\n"
                           "Column 3- the radius of the circle (pixels)\n"
                           "Column 4- the counts within a circular aperture having the radius in column 3\n" 
                           "Column 5- the instrumental magnitude\n"
                           "Column 6- the instrumental magnitude error\n" 
                           "Column 7- the significance (signal-to-noise)"); 
   
     
    th->setLabel(0,"x coord");
    _table->setText(0, 0, "2048.934");
 

    th->setLabel(1,"y coord");
    _table->setText(0, 1, "2048.934");
 
    th->setLabel(2,"radius");
    _table->setText(0, 2, "600.0");
 

    th->setLabel(3,"counts");
    _table->setText(0, 3, "645678.99");

    th->setLabel(4,"mag");
    _table->setText(0, 4, "17.99");

    th->setLabel(5,"+/-");
    _table->setText(0, 5, "17.99");

    th->setLabel(6,"Significance");
    _table->setText(0, 6, "17.9900000");


    /********************************************
    *
    * Adjust the size of the cells
    *
    *******************************************/
    QFont f = _table->font();     
    QFontMetrics fm( f );
 
    int width = 0;
   
   
    for (int i = 0; i < noColumns; i++)
    {         
        int columnWidth =  fm.width(_table->text(0, i)) + 12;

   
       _table->setColumnWidth(i, columnWidth);
	_table->setText(0, i, " ");
        width += columnWidth;
    }
  

 
    width += 55;
    _table->resize(width, _table->height());

    _table->setNumRows(0);   
 
    maxWidth = max( maxWidth, _table->geometry().right() ); // The maximum width of the window

  


 
    _deleteListBtn = new OmSourceButton(this, "&delete", _table->geometry().left(), _table->geometry().bottom() + 20, 80, 30,
               "Click here to delete the source list");
  

  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );

    _deleteListBtn->setDisabled( true );



    //    group->resize( _deleteListBtn->geometry().right() + 20, _deleteListBtn->geometry().bottom() + 20); 

    maxWidth = max( maxWidth, _deleteListBtn->geometry().right()); // The maximum width of the window



  


    OmSourceGroupBox *colourBox = new OmSourceGroupBox( this,  "Ds9 image display", _deleteListBtn->geometry().left(),
            _deleteListBtn->geometry().bottom() + 20, 180, 70);





    /******************************************************************
    * A spin box to set the old regions colour
    *****************************************************************/


    OmSourceLabel *label1 = new OmSourceLabel(colourBox, "old regions", 10, 20);

    _spinBoxColour1 = new OmSpinBox( colourBox, "Change the colour of the old regions", OmSourceBrowser::_colourList, OmSourceBrowser::_oldRegionsNo);

    _spinBoxColour1->setGeometry( label1->geometry().left(), label1->geometry().bottom(), 70, 25);

    connect(_spinBoxColour1, SIGNAL(valueChanged(int)), this, SLOT ( oldRegionColourChanged(int)));



  

 
    /******************************************************************
    * A spin box to set the new regions colour
    *****************************************************************/
    
     OmSourceLabel *label2 = new OmSourceLabel( colourBox, "new regions", label1->geometry().right() + 30, label1->geometry().top());


    _spinBoxColour2 = new OmSpinBox( colourBox, "Change the colour of the new regions", OmSourceBrowser::_colourList.size() - 1,
                      OmSourceBrowser::_colourList, OmSourceBrowser::_newRegionsNo);
    _spinBoxColour2->setGeometry( label2->geometry().left(), label2->geometry().bottom(), 70, 25);
    connect(_spinBoxColour2, SIGNAL(valueChanged(int)), this, SLOT ( newRegionColourChanged(int)));

 

    _spinBoxColour2->show();
  
 

    /******************************************************************
    * A spin box to set image update time
    *****************************************************************/
    
     OmSourceLabel *label3 = new OmSourceLabel( colourBox, "update time", label2->geometry().right() + 30, label1->geometry().top());


    _spinBoxUpdate = new QSpinBox( colourBox, "Change the ds9 image update time (secs)");
    _spinBoxUpdate->setGeometry( label3->geometry().left(), label3->geometry().bottom(), 70, 25);
    connect(_spinBoxUpdate, SIGNAL(valueChanged(int)), this, SLOT ( updateTimeChanged(int)));

    QToolTip::add(_spinBoxUpdate, 
         "The time interval (secs) at which the ds9 image will be updated with the new regions.\n"
         "Please note that during the time when the ds9 image is being updated any sources selected will be ignored");
 
    
    _spinBoxUpdate->show();
  
    _spinBoxUpdate->setMinValue( 1 );
    _spinBoxUpdate->setMaxValue( 30 );
    
    _spinBoxUpdate->setValue( 3 );
   


    colourBox->resize( _spinBoxUpdate->geometry().right() + 20, _spinBoxUpdate->geometry().bottom() + 20 );

 

    maxWidth = max (maxWidth, colourBox->geometry().right() );

  




     /***********************************************************
     * Spin box to get the required aperture radius
     **********************************************************/
    int xPos = 20;
    int yPos = colourBox->geometry().bottom() + 20;
    int height = 30;

    OmSourceGroupBox *groupBox = new OmSourceGroupBox( this,  "Annulus (pixels)", xPos, yPos, 180, 70);
   
    xPos = 10;
    yPos = 20;
    _apertureTxt = new OmSourceLabel(groupBox, "Aperture",  xPos, yPos, height);
 
    xPos += 80; 
  
    _apertureSpinBox = new QSpinBox(groupBox, "SpinBox2" );
    _apertureSpinBox->setGeometry( xPos, yPos, 50, height ); 
    
    _apertureSpinBox->setMaxValue((int)_apertureRadius);
    _apertureSpinBox->setMinValue(3);
    _apertureSpinBox->setValue(50);





    connect(_apertureSpinBox, SIGNAL(valueChanged(int)), this, SLOT(apertureRadiusHasChanged(int)));

    QString text;    
    text.sprintf("Change the photometry aperture radius\n (allowed values=%d-%d)", 
		 _apertureSpinBox->minValue(), _apertureSpinBox->maxValue());
    QToolTip::add(_apertureSpinBox, text);

  
  
    _centroidButton = new QCheckBox("Centroiding", groupBox);
        QToolTip::add(_centroidButton, "If checked then the selected points will be centroided"); 
   

    _centroidButton->setGeometry(_apertureSpinBox->geometry().right() + 20, yPos, 100, 30);
    _centroidButton->setChecked(true); 

    groupBox->resize(_centroidButton->geometry().right() + 20,_centroidButton->geometry().bottom() + 20); 

    maxWidth = max (maxWidth, groupBox->geometry().right() );


 






    /*****************************************************************************
    * A help button
    *****************************************************************************/

    yPos = groupBox->geometry().bottom() + 20;
    xPos = 20;
    OmSourceButton *helpButton = new OmSourceButton(this, " &Help ", xPos, yPos, 30, 
               "Click here for help");
  
  
    connect( helpButton, SIGNAL(clicked()), this, SLOT ( showHelpInformation()) );
   
    QToolTip::add( helpButton, "Click here, or click the right mouse-button,\n""for information on the photometry, etc");
  




    xPos = helpButton->geometry().right() + 20;

    _okBtn = new OmSourceButton(this, "&Okay",
                            "Click here when you have finished selecting sources\n"
		            "and wish to add them to the source-list.\n"
                            "Aperture photometry will be performed on each source-\n"
                            "using the selected aperture radius\n"
                            "If a large number of sources have been selected, or the\n"
		            "image is large, this may take some time!"); 



    _okBtn->setGeometry(xPos, yPos, 80, 30);
    connect(_okBtn, SIGNAL( clicked() ), this, SLOT( accept1() ));

    xPos = _okBtn->geometry().right() + 10;


    _cancelBtn = new OmSourceButton( this, "&Cancel",
                            "Click here when you have finished selecting sources\n"
		            "but do not wish to add them to the source-list");
  
 

    _cancelBtn->setGeometry( xPos, yPos, 80, 30);
    connect(_cancelBtn, SIGNAL( clicked() ), this, SLOT( cancel1() ));
   
  
    //  cout << "addsources line 277\n";


    OmImageRegions::_noBackgroundRegions = 0;
  
    update();
 

    // Initialise some photometry arrays
    F90MODPROC(omsource_glue, initialisephotometry)();
 



    resize( maxWidth + 10, _cancelBtn->geometry().bottom() + 20);


    OmSourceRegions *points1 = OmImageRegions::GetNewRegions(_imageDisplay, _nx, _ny);
     
    if (points1)
        delete points1;

    if ( _oldSources) 
    {
       
        _spinBoxColour1->show();
    }
    else
    {
     
        _spinBoxColour1->setDisabled( true );
    }

   



    _timer = new QTimer( this);


    connect( _timer, SIGNAL(timeout()), this, SLOT(updateRegions()) );
  
    _timer->start( _time ); // 5 seconds single-shot timer
    _imageDisplay->send("cmap heat");     
         
    _regionTimer = new QTimer( this);


    connect( _regionTimer, SIGNAL(timeout()), this, SLOT(redrawRegions()) );
 


 

};
/***************************************
 * Destructor
 **************************************/
AddSourcesDlg::~AddSourcesDlg()
{

    _timer->stop();
    delete _timer;
    delete _regionTimer;

}


/****************************************************
 *
 ****************************************************/
void AddSourcesDlg::disableButtons()
{


    _deleteListBtn->setDisabled( true );

   _okBtn->setDisabled( true );
   _cancelBtn->setDisabled( true );

 



   _spinBoxColour1->setDisabled( true );
   _spinBoxColour2->setDisabled( true );
   _spinBoxUpdate->setDisabled ( true );







    qApp->processEvents();
 

}


/****************************************************
 *
 ****************************************************/
void AddSourcesDlg::enableButtons()
{


    _deleteListBtn->setEnabled( true );

   _okBtn->setEnabled( true );
   _cancelBtn->setEnabled( true );
  

   if( _oldSources)
       _spinBoxColour1->setEnabled( true );


    _spinBoxColour2->setEnabled( true );






   _spinBoxUpdate->setEnabled ( true );

    qApp->processEvents();
 

}




/****************************************************
 *
 ****************************************************/
void AddSourcesDlg::cancel1()
{

    disableButtons();

 
  


    _timer->stop();

    deleteList();  // Delete the newly selected regions
    reject();
}
/****************************************************
 *
 ****************************************************/
void AddSourcesDlg::accept1()
{
  

    disableButtons();
    _timer->stop();

  

    if (OmImageRegions::_noBackgroundRegions == 0) 
    {
        reject();
        return;
    }
 

   
   




    F90MODPROC(omsource_glue, appendtosourceregions)(OmImageRegions::_backgroundRegions);
    
    // Need to compute the parameters for each new source
     
    if (_centroidButton->isChecked())  // Centroiding switched on
        F90MODPROC(omsource_glue, addregions1)(OmImageRegions::_backgroundRegions);
    else
        F90MODPROC(omsource_glue, addregions2)(OmImageRegions::_backgroundRegions);
 
 





    // Get the newly added sources 
    OmSourceParameters& sources = F90MODVAR(omsource_glue, myaddedsources);
    
 

    for (unsigned i = 1; i <= _radiusList.size(); i++)
    {
        sources[i].photometryradius = _radiusList[i - 1];

    }


    // Display the dialogue to do the photometry
    QString caption =  "Computing photometry- please wait";



    
    ProcessDlg *processDlg = new ProcessDlg(this, &sources, caption, 
            ProcessDlg::aperturePhotometry);
   

    processDlg->exec();



  
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    int j = mySources.ubound() - _radiusList.size();



    for (unsigned i = 1; i <= _radiusList.size(); i++)
    {

     
        mySources[j + i] = sources[i];

    }    

    F90MODPROC(omsource_glue, deletebackgroundlist)();


   

    delete(OmImageRegions::_backgroundRegions);

    OmImageRegions::_backgroundRegions = 0;
   
    OmImageRegions::_noBackgroundRegions = 0;

    accept(); 
}
/**********************************************
 * Deletes the list of selected sources
 *********************************************/
void AddSourcesDlg::deleteList()
{


   disableButtons();

   _deleteListBtn->setDisabled( true );


   _timer->stop();

   //   cout << "Delete list line 312\n";
    // Delete all the regions        
    _imageDisplay->send( "regions deleteall" );
	
    // Reset the table  
    _table->setNumRows(0);   
  
    _deleteListBtn->setDisabled(TRUE);
    // Delete the background regions

    //  cout << "Delete list line 322\n";

  
    F90MODPROC(omsource_glue, deletebackgroundlist)();
    F90MODPROC(omsource_glue, deletebackgroundregions)();


    // cout << "Delete list line 329\n";


    delete (OmImageRegions::_backgroundRegions);
    OmImageRegions::_backgroundRegions = 0;
    OmImageRegions::_noBackgroundRegions = 0; 




    delete (OmImageRegions::_oldRegions);
    OmImageRegions::_oldRegions = 0;
    OmImageRegions::_noRegions = 0;
   


    //  cout << "Delete list line 336\n";


    _radiusList.clear();
   


    _imageDisplay->displayCurrentSourceList( OmSourceBrowser::_oldRegionsColour.c_str() );





    //   cout << "Delete list line 345\n";

    OmSourceRegions *points1 = OmImageRegions::GetNewRegions( _imageDisplay, _nx, _ny );
    if (points1) 
    {
        delete(points1);	  
        points1 = 0;
    }

    enableButtons();


    _timer->start( _time );
 
     
}
/************************************************************
*
************************************************************/

//void AddSourcesDlg::mousePressEvent( QMouseEvent *e )
//{
//    QApplication::beep();
//    if ( e->button() == QMouseEvent::LeftButton )               // left button pressed
//    QMessageBox::critical( 0, "omsource",
//			       QString("LEFT MOUSE BUTTON")); 

//}
/*********************************************************
* Checks the image for any new background regions
* and, if necessary, updates the image and table
********************************************************/
void AddSourcesDlg::updateRegions()
{ 

 
    _timer->stop();

   

    //    cout << "updateregions line 651\n";


    // Get the new regions taht have been selected by the user
    OmSourceRegions *newRegions = OmImageRegions::GetNewRegions( _imageDisplay, _nx, _ny );
    
  
    if (newRegions == 0) 
    {
        _timer->start( _time );
 

	//	cout << "updateregions Line 664 no new regions nobackgroundregions=" <<   OmImageRegions::_noBackgroundRegions  << endl;
        return;
    }
           
    _regionTimer->stop();




   disableButtons();  

   //   cout << "Line 671 updateregions new regions nobackgroundregions=" <<   OmImageRegions::_noBackgroundRegions  << endl;


   // Compute the counts for each newly selected region     

    for (int i = newRegions->lbound(); i <= newRegions->ubound(); i++)
    {   
 

        float x = (*newRegions)[i].x();
        float y = (*newRegions)[i].y();
       
        float counts = F90MODPROC(omsource_glue, getimagecounts)(x, y, _apertureRadius);
	

       	OmSourceParameter theSource;
        theSource.xcoord = x;
        theSource.ycoord = y;
        theSource.photometryradius = _apertureRadius;
        theSource.rawbackgroundcountrate = -1.0;

	//	cout << "OmSourceAddSourcesDlg line 786 photometryradius=" <<  _apertureRadius << endl;

        /*************************************************************************
	*
        * Check to see if supposed to centroid the regions
        *
        *************************************************************************/

        if (_centroidButton->isChecked())  // Centroiding switched on
	{  

	  // cout << "OmSourceAddSourcesDlg.cc line 798- supposed to be centroiding sources\n";

            F90MODPROC(omsource_glue, computesourcepositions2)(theSource);
            x = theSource.xcoord;
            y = theSource.ycoord;

	    //            newRegions[i].x() = theSource.xcoord;
	    // newRegions[i].y() = theSource.ycoord;
             

        }


        
	//cout << "L812 calling computephotometry2\n"; 
        F90MODPROC(omsource_glue, computephotometry2)(theSource);
      
	//cout << "L815 OmSourceAddSourcesDlg source%photometryradius=" << theSource.photometryradius << endl;

        /*******************************************************************
	*
	* Update the table
        *
        ********************************************************************/
        updateTable(x, y, _apertureRadius, counts, theSource);  
        _radiusList.push_back(_apertureRadius);
    

   
 
   }

  

   _deleteListBtn->setEnabled( true );


   //   cout << "Line 693 updateregions ubound newregions=" << newRegions->ubound() << endl;
   //   cout << "ubound new regions=" << newRegions->ubound() << endl;

    OmImageRegions::updateBackgroundRegions( newRegions );
 

    //    cout << "Line 700 updateregions new regions nobackgroundregions=" <<   OmImageRegions::_noBackgroundRegions  << endl;


    delete( newRegions );  // Finished with the new background regions
         


    _imageDisplay->displayCurrentSourceList( OmSourceBrowser::_oldRegionsColour.c_str() );

   
    if ( OmImageRegions::_noBackgroundRegions > 0 )
    {

      //    cout << "Line 719 updateregions\n";

        _imageDisplay->displaySourceRegions(OmImageRegions::_backgroundRegions, _radiusList, OmSourceBrowser::_newRegionsColour.c_str());

    }

   
    //    cout << "Line 732 updateregions new regions nobackgroundregions=" <<  OmImageRegions::_noBackgroundRegions  << endl;

     _timer->start( _time );
    enableButtons();  

}

/*****************************************************
 * Called when the value of apertureradius 
 *spin-box has changes
 ****************************************************/
void AddSourcesDlg::apertureRadiusHasChanged(int radius)
{
    _apertureRadius = radius;
    _apertureSpinBox->setValue(radius);
    
}
/***********************************************************
*
*
***********************************************************/
void AddSourcesDlg::updateTable(float x, float y, float radius, 
           float counts, OmSourceParameter &source)
{
    int rowNo = _table->numRows();
    
    _table->setNumRows(rowNo + 1);
    
    QString text;
                 
    text.sprintf("%6.2f", x);
    _table->setText(rowNo, 0, text);

    text.sprintf("%6.2f", y);
    _table->setText(rowNo, 1, text);

    text.sprintf("%2d",  (int)radius);
    _table->setText(rowNo, 2, text);

    text.sprintf("%8.2f", counts);  
    _table->setText(rowNo, 3, text);

  
    text.sprintf("%5.2f", source.magnitude);  
    _table->setText(rowNo, 4, text);

  
    text.sprintf("%5.2f", source.magnitudeerror);  
    _table->setText(rowNo, 5, text);

    text.sprintf("%5.2f", source.significance);  
    _table->setText(rowNo, 6, text);

    _deleteListBtn->setEnabled(TRUE);

}
/***********************************************************
*
* Loads all the new sources into the table
*
***********************************************************/
void AddSourcesDlg::update()
{

 
    if (OmImageRegions::_noBackgroundRegions == 0) return;
  
    _imageDisplay->displayCurrentSourceList( OmSourceBrowser::_oldRegionsColour.c_str());

    _imageDisplay->displaySourceRegions(OmImageRegions::_backgroundRegions, _radiusList, OmSourceBrowser::_newRegionsColour.c_str());
 
    // Compute the counts for each newly selected region
  

    for (int i = OmImageRegions::_backgroundRegions->lbound(); 
           i <= OmImageRegions::_backgroundRegions->ubound(); i++)
    {   

        float x = (*OmImageRegions::_backgroundRegions)[i].x();
        float y = (*OmImageRegions::_backgroundRegions)[i].y();
       

        // Get the counts           
	float counts = F90MODPROC(omsource_glue, getimagecounts)(x, y, _radiusList[i - 1]);     

        OmSourceParameter theSource;
        theSource.xcoord = x;
        theSource.ycoord = y;
        theSource.photometryradius = _radiusList[i - 1];
        theSource.rawbackgroundcountrate = -1.0;  
            
        if (_centroidButton->isChecked())  // Centroiding switched on
	{  
            F90MODPROC(omsource_glue, computesourcepositions2)(theSource);
        }
         
    
	cout << "L958 calling computephotometry2\n"; 
        F90MODPROC(omsource_glue, computephotometry2)(theSource);
               
          
        // Update the table
        updateTable(x, y, _radiusList[i - 1], counts, theSource);  	

    }

    _deleteListBtn->setEnabled(TRUE);

} 

/****************************************************************************
*
*
*
*
****************************************************************************/
void AddSourcesDlg::oldRegionColourChanged( int no )
{


    _regionTimer->stop();

    _timer->stop();

    OmSourceBrowser::_oldRegionsColour = OmSourceBrowser::_colourList[no - 1];

    OmSourceBrowser::_oldRegionsNo = no;

    _timer->start( _time );

    _regionTimer->start( 2000 ); // 5 seconds single-shot timer

}


/****************************************************************************
*
*
****************************************************************************/
void AddSourcesDlg::newRegionColourChanged( int no )
{



    _regionTimer->stop();
    _timer->stop();

    OmSourceBrowser::_newRegionsColour = OmSourceBrowser::_colourList[no - 1];

    OmSourceBrowser::_newRegionsNo = no;

    _timer->start( _time );


    if (OmImageRegions::_noBackgroundRegions == 0) return;

    _regionTimer->start( 2000 ); // 5 seconds single-shot timer



}

/****************************************************************************
*
*
****************************************************************************/
void AddSourcesDlg::updateTimeChanged( int time )
{

    QString text1;
    text1.sprintf("%2d", time);
    QString text( QString("Please select new sources by clicking the left\n")    +
                  QString("mouse-button. The selected sources can be deleted\n") +
                  QString("by clicking the 'Delete' button. The selected\n")     +
                  QString("sources will be displayed in ")                       +
                  QString( OmSourceBrowser::_newRegionsColour.c_str() )          +
                  QString(".\nThe DS9 image will be updated every ")             +
                  text1 + QString("\nseconds, approximately.") );


    _label->setText( text, 60 );

    _time = time * 1000;
   

    if ( _timer)
    {
        _timer->stop();
        _timer->start( _time );
    }
  
}
/**************************************************************************************
*
*
*
***************************************************************************************/
void AddSourcesDlg::redrawRegions()
{

    _regionTimer->stop();
    _imageDisplay->displayCurrentSourceList( OmSourceBrowser::_oldRegionsColour.c_str() );

    if ( OmImageRegions::_noBackgroundRegions > 0 )
    {
        _imageDisplay->displaySourceRegions(OmImageRegions::_backgroundRegions, _radiusList, OmSourceBrowser::_newRegionsColour.c_str());
    }
}
/******************************************************
*
******************************************************/
    
void AddSourcesDlg::showHelpInformation()
{
 

    static const char* information[] = {
  
    "<b>SELECTING NEW SOURCES FROM THE IMAGE</b><br>"
    "<hr><br><br>"
    "<b>1)</b> A source is selected by clicking the left mouse-button at a particular point."
    " After a certain interval of time (\"<b>the update time </b>\"), the image "
    " will be updated to display the newly selected sources in the \"<b>new regions</b>\" colour. <br><br>" 
                    
    "<b>2)</b> When you have finished selecting new sources, press either the \"<b>Okay</b>\" or \"<b>Cancel</b>\" button. <br>"
    "In the case of the former, the newly selected sources will be added to the list of sources. If the \"<b>Centroiding</b>\""
    " is set, then the source positions will be refined by centroiding. Aperture photometry will then be conducted on these"
    " sources using the aperture set in the \"<b>Aperture</b\" spin box",   
     
    0};     
                    
 

    HelpDlg *helpDlg = new HelpDlg( this, information);


    helpDlg->exec();
         
    delete helpDlg;  
 
}
/*************************************************************
*
************************************************************/
void AddSourcesDlg::mousePressEvent( QMouseEvent* event)
{
  
    if ( event->button() == RightButton )
    {   

        showHelpInformation();
    }



}


